#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `blanketdb` module."""


import unittest
from datetime import datetime, timedelta

from webtest import TestApp

from blanketdb import BlanketDB


class TestBlanketdb(unittest.TestCase):
    """Tests for `blanketdb` package."""

    def setUp(self):
        """Set up test instance of `BlanketDB`"""
        self.next_date = datetime(2022, 7, 15)
        self.db = BlanketDB(':memory:', lambda: self.next_date)
        self.app = TestApp(self.db)

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_storing_from_python(self):
        '''Test storing of data using Python API'''
        data = dict(a=1, b=2)
        stored = self.db.store(data)
        self.assertEqual(data, stored['data'])
        self.assertEqual(data, self.db[stored['id']]['data'])
        self.assertEqual(1, len(list(self.db)))
        self.assertEqual(data, list(self.db)[0]['data'])
        stored2 = self.db.store_dict(x='test')
        self.assertEqual(dict(x='test'), stored2['data'])
        self.assertEqual(dict(x='test'), self.db[stored2['id']]['data'])
        self.assertEqual(2, len(list(self.db)))
        not_in_db_id = abs(stored['id']) + abs(stored2['id']) + 1
        self.assertEqual(None, self.db[not_in_db_id])
        del self.db[stored['id']]
        self.assertEqual(None, self.db[stored['id']])

    def test_query_from_python(self):
        '''Test `BlanketDB.query` method using Python API'''
        for i in range(10):
            resp = self.db.store_dict(bucket='testbucket', number=i)
            if i == 0:
                self.assertEqual(1, resp['id'], 'SQLite id changed unexpected')
            if i == 4:
                after_four = self.next_date
            self.next_date += timedelta(seconds=4)
        self.assertEqual(10, len(list(self.db.query(bucket='testbucket'))))
        self.assertEqual(0, len(list(self.db.query(bucket='testbucket2'))))
        self.assertEqual(10, len(list(self.db.query(since_id=1))))
        self.assertEqual(9, len(list(self.db.query(since_id=2))))
        self.assertEqual(8, len(list(self.db.query(since_id=3))))
        self.assertEqual(2, len(list(self.db.query(before_id=3))))
        self.assertEqual(6, len(list(self.db.query(since=after_four))))
        self.assertEqual(4, len(list(self.db.query(before=after_four))))
        self.assertEqual(2,
                         len(list(self.db.query(before=after_four, limit=2))))
        i = 10
        for entry in self.db.query(newest_first=True):
            self.assertGreater(i, entry['data']['number'])
            i = entry['data']['number']
        i = -1
        for entry in self.db.query(newest_first=False):
            self.assertLess(i, entry['data']['number'])
            i = entry['data']['number']

    def test_delete_from_python(self):
        '''Test `BlanketDB.delete` method using Python API'''
        for i in range(10):
            resp = self.db.store_dict(bucket='testbucket', number=i)
            if i == 0:
                self.assertEqual(1, resp['id'], 'SQLite id changed unexpected')
            if i == 4:
                after_four = self.next_date
            self.next_date += timedelta(seconds=4)
        self.db.delete(since=after_four)
        self.assertEqual(4, len(list(self.db)))
        self.db.delete(before=after_four)
        self.assertEqual(0, len(list(self.db)))
        self.db.store_dict(bucket='testbucket', irrelevant='data')
        self.db.store_dict(bucket='testbucket2', irrelevant='data')
        self.db.delete(bucket='testbucket')
        self.assertEqual(1, len(list(self.db)))
        self.db.delete()
        self.assertEqual(0, len(list(self.db)))

    def test_basic_web_requests(self):
        '''Test basic web requests'''
        resp = self.app.get('/', status=200)
        self.assertEqual(0, len(resp.json['entries']))
        self.assertEqual(0, resp.json['number_of_entries'])
        self.app.get('/_entry/123', status=404)
        self.app.get('/_entry/', status=400)

    def test_method_not_allowed_requests(self):
        '''Test several method/route combinations which are not allowed'''
        self.app.post('/_entry', status=405)
        self.app.post('/_entry/123', status=405)
        self.app.put('/_entry/', status=405)
        self.app.put('/_entry/123', status=405)
        self.app.put('/anything', status=405)

    def test_create_entry_requests(self):
        '''Test entry creation by request'''
        self.app.post('/', status=201)
        self.app.post('/otherbucket', status=201)
        self.app.post('/', dict(a=1), status=201)  # post form
        self.app.post_json('/', dict(a=1), status=201)  # post json
        self.assertEqual(4, len(self.app.get('/', status=200).json['entries']))
        self.assertEqual(1, len(self.app.get('/otherbucket', status=200)
                                        .json['entries']))
        self.assertEqual(3, len(self.app.get('/default', status=200)
                                        .json['entries']))

    def test_query_requests(self):
        '''Test querying'''
        for i in range(10):
            resp = self.app.post('/testbucket', dict(number=i), status=201)
            if i == 0:
                self.assertEqual(1, resp.json['id'],
                                 'SQLite id changed unexpected')
            if i == 4:
                after_four = self.next_date
            self.next_date += timedelta(seconds=4)
        self.assertEqual(10, self.app.get('/testbucket', status=200)
                                     .json['number_of_entries'])
        self.assertEqual(0, self.app.get('/testbucket2', status=200)
                                    .json['number_of_entries'])
        self.assertEqual(10, self.app.get('/testbucket', dict(since_id=1),
                                          status=200)
                                     .json['number_of_entries'])
        self.assertEqual(9, self.app.get('/testbucket', dict(since_id=2),
                                         status=200)
                                    .json['number_of_entries'])
        self.assertEqual(8, self.app.get('/testbucket', dict(since_id=3),
                                         status=200)
                                    .json['number_of_entries'])
        self.assertEqual(2, self.app.get('/testbucket', dict(before_id=3),
                                         status=200)
                                    .json['number_of_entries'])
        self.assertEqual(6, self.app.get('/testbucket', dict(since=after_four),
                                         status=200)
                                    .json['number_of_entries'])
        self.assertEqual(4, self.app.get('/testbucket',
                                         dict(before=after_four),
                                         status=200)
                                    .json['number_of_entries'])
        self.assertEqual(2, self.app.get('/testbucket',
                                         dict(before=after_four, limit=2),
                                         status=200)
                                    .json['number_of_entries'])
        i = 10
        for entry in self.app.get('/', dict(newest_first=True), status=200) \
                             .json['entries']:
            self.assertGreater(i, entry['data']['number'])
            i = entry['data']['number']
        i = -1
        for entry in self.app.get('/', dict(newest_first=False), status=200) \
                             .json['entries']:
            self.assertLess(i, entry['data']['number'])
            i = entry['data']['number']

    def test_delete_requests(self):
        '''Test entry deletion'''
        # delete by id
        resp = self.app.post('/', status=201)
        id_created = resp.json['id']
        self.assertTrue(isinstance(id_created, int))
        self.app.get('/_entry/{0}'.format(id_created), status=200)
        self.app.delete('/_entry/{0}'.format(id_created), status=200)
        self.app.get('/_entry/{0}'.format(id_created), status=404)
        self.app.delete('/_entry/{0}'.format(id_created), status=404)
        # delete by bucket
        for i in range(3):
            self.app.post('/', status=201)
        for i in range(4):
            self.app.post('/otherbucket', status=201)
        self.assertEqual(3, self.app.get('/default', status=200)
                                    .json['number_of_entries'])
        self.app.delete('/default', status=200)
        self.assertEqual(0, self.app.get('/default', status=200)
                                    .json['number_of_entries'])
        self.assertEqual(4, self.app.get('/otherbucket', status=200)
                                    .json['number_of_entries'])
        self.app.delete('/otherbucket', status=200)
        self.assertEqual(0, self.app.get('/otherbucket', status=200)
                                    .json['number_of_entries'])
        self.app.delete('/otherbucket', status=200)  # still 200
        for i in range(3):
            self.app.post('/', status=201)
        for i in range(4):
            self.app.post('/otherbucket', status=201)
        self.assertEqual(7, self.app.get('/', status=200)
                                    .json['number_of_entries'])
        self.app.delete('/', status=200)  # deletes all, not default bucket
        self.assertEqual(0, self.app.get('/', status=200)
                                    .json['number_of_entries'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [ ]

setup_requirements = [ ]

test_requirements = [ ]

setup(
    author="luphord",
    author_email='luphord@protonmail.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="A simple HTTP accessible database for IoT projects",
    entry_points={
        'console_scripts': [
            'blanketdb=blanketdb:cli',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='blanketdb',
    name='blanketdb',
    py_modules=['blanketdb'],
    scripts=['blanketdb.py'],
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/luphord/blanketdb',
    version='0.3.1',
    zip_safe=False,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for helper functions"""


import unittest
import json

from datetime import datetime, date, timedelta
from blanketdb import _parse_form, _parse_dt, _serialize_json, _j


def is_close(dt1, dt2, max_diff_sec=10):
    return abs(dt1.timestamp() - dt2.timestamp()) < max_diff_sec


class TestHelperFunctions(unittest.TestCase):
    """Tests for helper functions"""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_parse_form(self):
        '''Test function for parsing and type converting forms'''
        self.assertEqual(dict(a='b'), _parse_form('a=b'))
        self.assertEqual(dict(a='b', c='d'), _parse_form('a=b&c=d'))
        self.assertEqual(dict(a=1), _parse_form('a=1'))
        self.assertEqual(1.1, _parse_form('a=1.1')['a'])
        self.assertEqual(float, type(_parse_form('a=1.0')['a']))
        self.assertEqual(dict(a=True), _parse_form('a=true'))
        self.assertEqual(dict(a=False), _parse_form('a=false'))

    def test_parse_date(self):
        '''Test function for customized date parsing'''
        self.assertEqual('', _parse_dt(''))
        # this function only parses dates in custom format
        self.assertEqual('2025-02-03', _parse_dt('2025-02-03'))
        self.assertEqual('', _parse_dt(None))
        self.assertEqual(date.today(), _parse_dt('today'))
        self.assertEqual(date.today() - timedelta(days=1),
                         _parse_dt('yesterday'))
        # days
        self.assertTrue(is_close(datetime.now() - timedelta(days=2),
                        _parse_dt('2d')))
        self.assertTrue(is_close(datetime.now() - timedelta(days=2),
                        _parse_dt('2days')))
        self.assertTrue(is_close(datetime.now() - timedelta(days=2),
                        _parse_dt('2 days')))
        self.assertTrue(is_close(datetime.now() - timedelta(days=1),
                        _parse_dt('1day')))
        # hours
        self.assertTrue(is_close(datetime.now() - timedelta(hours=2),
                        _parse_dt('2h')))
        self.assertTrue(is_close(datetime.now() - timedelta(hours=2),
                        _parse_dt('2hours')))
        self.assertTrue(is_close(datetime.now() - timedelta(hours=2),
                        _parse_dt('2 hours')))
        self.assertTrue(is_close(datetime.now() - timedelta(hours=1),
                        _parse_dt('1hour')))
        # minutes
        self.assertTrue(is_close(datetime.now() - timedelta(minutes=2),
                        _parse_dt('2m')))
        self.assertTrue(is_close(datetime.now() - timedelta(minutes=2),
                        _parse_dt('2min')))
        self.assertTrue(is_close(datetime.now() - timedelta(minutes=2),
                        _parse_dt('2 min')))
        # seconds
        self.assertTrue(is_close(datetime.now() - timedelta(seconds=20),
                        _parse_dt('20s')))
        self.assertTrue(is_close(datetime.now() - timedelta(seconds=20),
                        _parse_dt('20sec')))
        self.assertTrue(is_close(datetime.now() - timedelta(seconds=20),
                        _parse_dt('20 sec')))

    def test_serialize_json(self):
        '''Test function for serializing json with dates'''
        self.assertEqual('{}', _serialize_json({}))
        self.assertEqual('"2030-01-02"', _serialize_json(date(2030, 1, 2)))
        self.assertEqual('"2030-01-02T03:04:05"',
                         _serialize_json(datetime(2030, 1, 2, 3, 4, 5)))

    def test_j(self):
        '''Test function for serializing to json and encoding bytes'''
        self.assertEqual(b'{}', _j({}))
        self.assertEqual(b'"2030-01-02"', _j(date(2030, 1, 2)))
        # avoid indent comparison by decoding json again
        self.assertEqual(dict(a=2, b=3),
                         json.loads(_j(a=2, b=3).decode('utf8')))
